"""
Define some generators based on the caching mechanism that
are commonly needed for code generation
"""

from dune.perftool.generation import generator_factory
from dune.perftool.cgen.clazz import AccessModifier

symbol = generator_factory(item_tags=("symbol",))
preamble = generator_factory(item_tags=("preamble",), counted=True)


def include_file(include, filetag=None):
    assert filetag
    from cgen import Include
    gen = generator_factory(on_store=lambda i: Include(i), item_tags=(filetag, "include"), no_deco=True)
    return gen(include)


def initializer_list(obj, params, classtag=None):
    assert classtag
    gen = generator_factory(item_tags=(classtag, "initializer"), counted=True, no_deco=True, cache_key_generator=lambda *a: a[0])
    return gen("{}({})".format(obj, ", ".join(params)))


def base_class(baseclass, classtag=None, access=AccessModifier.PUBLIC, construction=[]):
    assert classtag

    from dune.perftool.cgen.clazz import BaseClass
    gen = generator_factory(item_tags=("baseclass", classtag), on_store=lambda n: BaseClass(n, inheritance=access), counted=True, no_deco=True)

    if construction:
        initializer_list(baseclass, construction, classtag=classtag)

    return gen(baseclass)


def class_member(classtag=None, access=AccessModifier.PRIVATE):
    assert classtag
    from cgen import Value
    from dune.perftool.cgen.clazz import ClassMember

    return generator_factory(item_tags=(classtag, "member"), on_store=lambda m: ClassMember(m, access=access), counted=True)


def constructor_parameter(_type, name, classtag=None, constructortag="default"):
    assert classtag
    assert constructortag
    from cgen import Value

    gen = generator_factory(item_tags=(classtag, constructortag, "constructor_param"), counted=True, no_deco=True)
    return gen(Value(_type, name))


def template_parameter(classtag=None):
    assert classtag

    return generator_factory(item_tags=(classtag, "template_param"), counted=True)


def class_basename(classtag=None):
    assert classtag

    return generator_factory(item_tags=(classtag, "basename"))
